#include <glib.h>

#include "config.h"

#include "boilerplate-log.h"

static const gchar *
log_level_to_string (GLogLevelFlags level)
{
    switch (level)
    {
        case G_LOG_LEVEL_ERROR: return "ERROR";
        case G_LOG_LEVEL_CRITICAL: return "CRITICAL";
        case G_LOG_LEVEL_WARNING: return "WARNING";
        case G_LOG_LEVEL_MESSAGE: return "MESSAGE";
        case G_LOG_LEVEL_INFO: return "INFO";
        case G_LOG_LEVEL_DEBUG: return "DEBUG";
        default: return "UNKNOWN";
    }
}

static const gchar *
log_level_to_color (GLogLevelFlags level)
{
    // "\x1B[37m" WHITE
    switch (level)
    {
        case G_LOG_LEVEL_ERROR:    return "\x1B[31m"; // RED
        case G_LOG_LEVEL_CRITICAL: return "\x1B[33m"; // YELLOW
        case G_LOG_LEVEL_WARNING:  return "\x1B[35m"; // MAGENTA
        case G_LOG_LEVEL_MESSAGE:  return "\x1B[32m"; // GREEN
        case G_LOG_LEVEL_INFO:     return "\x1B[36m"; // CYAN
        case G_LOG_LEVEL_DEBUG:    return "\x1B[34m"; // BLUE
        default:                   return "\x1B[0m";  // RESET
    }
}

//TODO: Clean logging v3.0
void
log_handler (const gchar    *domain,
             GLogLevelFlags  log_level,
             const gchar    *message,
             gpointer        user_data)
{
    const gchar *log_level_string;
    const gchar *log_level_color;
    GDateTime   *date     = NULL;
    gchar       *time_fmt = NULL;

    if (DEBUG == FALSE && (log_level &G_LOG_LEVEL_DEBUG))
    {
        return;
    }

    /* Get log level & color */
    log_level_string = log_level_to_string (log_level &G_LOG_LEVEL_MASK);
    log_level_color = log_level_to_color (log_level &G_LOG_LEVEL_MASK);

    /* Time of log */
    date = g_date_time_new_now_utc ();
    time_fmt = g_date_time_format (date, "%H:%M:%S");

    /* Disable color mode, if user requests */
    if (DEBUG_COLOR == FALSE)
    {
        g_print ("[%s] %s - %s: %s\n",
                 time_fmt,
                 domain ? domain : PACKAGE,
                 log_level_string,
                 message);
    }
    else
    {
        g_print ("%s[%s] %s - %s: %s\x1B[0m\n",
                 log_level_color,
                 time_fmt,
                 domain ? domain : PACKAGE,
                 log_level_string,
                 message);
    }

    g_free (time_fmt);
    g_date_time_unref (date);
}
